/* Copyright (C) 2013-2021 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#define _POSIX_C_SOURCE 200112L /* strtok_r support */

#include "cstr_to_list.h"
#include "str.h"

#define CSTR_LIST_TYPE double
#include "cstr_to_list.h"

#define CSTR_LIST_TYPE float
#include "cstr_to_list.h"

#define CSTR_LIST_TYPE unsigned
#define CSTR_LIST_SUFFIX uint
#include "cstr_to_list.h"

RSYS_API res_T
cstr_parse_list
  (const char* str,
   const char delimiter,
   res_T (*parse_element)(const char* elmt, void* ctx),
   void* ctx) /* User defined data sent to 'parse_element' */
{
  struct str buf;
  char delim[2] = {'\0', '\0'};
  char* tk;
  char* tk_ctx;
  res_T res = RES_OK;

  str_init(NULL, &buf);

  if(!str || !parse_element) {
    res = RES_BAD_ARG;
    goto error;
  }

  /* Copy str in a temporary buffer to parse */
  res = str_set(&buf, str);
  if(res != RES_OK) goto error;

  /* Parse the list */
  delim[0] = delimiter;
  tk = strtok_r(str_get(&buf), delim, &tk_ctx);
  while(tk) {
    res = parse_element(tk, ctx);
    if(res != RES_OK) goto error;
    tk = strtok_r(NULL, delim, &tk_ctx);
  }

exit:
  str_release(&buf);
  return res;
error:
  goto exit;
}

